/*
 * Copyright (c) 2003 Xdvik project team
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * PAUL VOJTA OR ANY OTHER AUTHOR OF THIS SOFTWARE BE LIABLE FOR ANY CLAIM,
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */

#include "xdvi-config.h"
#include "xdvi.h"

#include "dvi-init.h" /* for total_pages */
#include "dvi-draw.h"

#include "search-dialog.h"
#include "dvi-search.h"
#include "message-window.h"
#include "statusline.h"

struct search_info_t {
    Boolean have_match;
    int match_page;
    int from_pos;
    int to_pos;
};

static void
scan_page(int from_page, int to_page, char **buf)
{
    UNUSED(from_page);
    UNUSED(to_page);
    UNUSED(buf);
}

static void
normalize_text(const char *buf, char **normalized_buf, const struct search_settings_t *settings)
{
    UNUSED(buf);
    UNUSED(normalized_buf);
    UNUSED(settings);
}

static void
try_match(const char *buf, const struct search_settings_t *settings, struct search_info_t *searchinfo)
{
    UNUSED(buf);
    UNUSED(settings);
    UNUSED(searchinfo);
}

static void
highlight_match(const struct search_info_t *searchinfo)
{
    UNUSED(searchinfo);
}

void
dvi_search(XtPointer arg)
{
    /* so that it survives between the callbacks; only
       reinitialized if settings->new_search is True (i.e.
       search had been cancelled, then restarted)
    */
    static struct search_info_t searchinfo = { False, 0, 0, 0 };
    struct search_settings_t *settings = (struct search_settings_t *)arg;
    static int scanned_page = -1;
    static int page_to_scan;
    static char *normalized_buf = NULL;

    if (raise_message_windows()) { /* any popups user still needs to deal with? */
	TRACE_GUI((stderr, "Still open message windows to deal with, returning ..."));
	return;
    }
    
    fprintf(stderr, "++++ dvi_search: Searching for |%s|\n", settings->term);
    fprintf(stderr, "++++   settings: down = %d, re = %d, case = %d, new = %d\n",
	    settings->direction, settings->use_regexp, settings->match_case, settings->new_search);

    page_to_scan = settings->from_page;
    
    searchinfo.have_match = False;

    if (settings->new_search) { /* re-initialize */
	searchinfo.match_page  = searchinfo.from_pos = searchinfo.to_pos = 0;
	scanned_page = -1;
    }

    while (!searchinfo.have_match && page_to_scan <= total_pages) {
	fprintf(stderr, "+++ CHECKING page %d\n", page_to_scan);
	if (scanned_page < page_to_scan) {
	    char *buf = NULL;
	    free(normalized_buf);
	    print_statusline(STATUS_SHORT, "Searching page %d ...", page_to_scan);
	    fprintf(stderr, "+++ SCANNING page %d\n", page_to_scan);
	    scan_page(page_to_scan, page_to_scan, &buf);
	    fprintf(stderr, "+++ NORMALIZING text\n");
	    normalize_text(buf, &normalized_buf, settings);
	    free(buf);
	}
	scanned_page = page_to_scan;
	page_to_scan++;

	fprintf(stderr, "+++ TRYING MATCH FROM %d\n", searchinfo.from_pos);
	try_match(normalized_buf, settings, &searchinfo);
    }

    if (searchinfo.have_match) {
	highlight_match(&searchinfo);
	searchinfo.from_pos = searchinfo.to_pos + 1;
    }
    else {
	if (settings->first_pass && settings->from_page > 0) {
	    settings->first_pass = false;
	    positioned_choice_dialog(NULL,
				     settings->x_pos, settings->y_pos,
				     "Yes", dvi_search, (XtPointer)settings,
				     /*
				       do nothing if user selects `cancel'; this will return to the
				       find dialog, as in Acroread (users may want to change the
				       search term or the settings).
				     */
				     "Cancel", NULL, (XtPointer)NULL,
				     "Searched from page %d to the end "
				     "of the DVI file.\nRestart the search "
				     "from the beginning of the DVI file?",
				     settings->from_page + 1);
	}
	else {
	    positioned_popup_message(MSG_INFO,
				     settings->x_pos, settings->y_pos,
				     NULL,
				     "Search term \"%s\" not found in DVI file.", settings->term);
	}
    }
}
